@@ -1,41 +1,38 @@
-// Archive/Tar/Item.h
+// TarItem.h
 
 #ifndef __ARCHIVE_TAR_ITEM_H
 #define __ARCHIVE_TAR_ITEM_H
 
-#include <time.h>
-
-#include "Common/Types.h"
-#include "Common/MyString.h"
-
 #include "../Common/ItemNameUtils.h"
+
 #include "TarHeader.h"
 
 namespace NArchive {
 namespace NTar {
 
-class CItem
+struct CItem
 {
-public:
   AString Name;
+  UInt64 Size;
+
   UInt32 Mode;
   UInt32 UID;
   UInt32 GID;
-  UInt64 Size;
-  UInt32 ModificationTime;
-  char LinkFlag;
-  AString LinkName;
-  char Magic[8];
-  AString UserName;
-  AString GroupName;
-
-  bool DeviceMajorDefined;
+  UInt32 MTime;
   UInt32 DeviceMajor;
-  bool DeviceMinorDefined;
   UInt32 DeviceMinor;
 
-  bool IsDirectory() const 
-  {  
+  AString LinkName;
+  AString User;
+  AString Group;
+
+  char Magic[8];
+  char LinkFlag;
+  bool DeviceMajorDefined;
+  bool DeviceMinorDefined;
+
+  bool IsDir() const
+  {
     switch(LinkFlag)
     {
       case NFileHeader::NLinkFlag::kDirectory:
@@ -48,22 +45,23 @@ public:
     return false;
   }
 
-  bool IsMagic() const 
-  {  
+  bool IsMagic() const
+  {
     for (int i = 0; i < 5; i++)
       if (Magic[i] != NFileHeader::NMagic::kUsTar[i])
         return false;
     return true;
   }
+
+  UInt64 GetPackSize() const { return (Size + 0x1FF) & (~((UInt64)0x1FF)); }
 };
 
-class CItemEx: public CItem
+struct CItemEx: public CItem
 {
-public:
   UInt64 HeaderPosition;
-  UInt64 LongLinkSize;
-  UInt64 GetDataPosition() const { return HeaderPosition + LongLinkSize + NFileHeader::kRecordSize; };
-  UInt64 GetFullSize() const { return LongLinkSize + NFileHeader::kRecordSize + Size; };
+  unsigned LongLinkSize;
+  UInt64 GetDataPosition() const { return HeaderPosition + LongLinkSize + NFileHeader::kRecordSize; }
+  UInt64 GetFullSize() const { return LongLinkSize + NFileHeader::kRecordSize + Size; }
 };
 
 }}
