@@ -23,6 +23,8 @@ static const UINT kIconTypesResId = 100;
 
 #ifdef _WIN32
 #include "Windows/Registry.h"
+#else
+#include "Common/StringConvert.h"
 #endif
 
 using namespace NWindows;
@@ -40,19 +42,23 @@ static CSysString GetLibraryFolderPrefix()
   CSysString path = fullPath;
   int pos = path.ReverseFind(TEXT(CHAR_PATH_SEPARATOR));
   return path.Left(pos + 1);
-#else // FIXME
+  #else
   const char *p7zip_home_dir = getenv("P7ZIP_HOME_DIR");
   if (p7zip_home_dir == 0) p7zip_home_dir="./";
+#ifdef _UNICODE
+  return MultiByteToUnicodeString(p7zip_home_dir);
+#else
   return p7zip_home_dir;
+#endif
   #endif
 }
 
 #define kCodecsFolderName TEXT("Codecs")
 #define kFormatsFolderName TEXT("Formats")
-static TCHAR *kMainDll = TEXT("7z.dll");
+static const TCHAR *kMainDll = TEXT("7z.dll");
 
 #ifdef _WIN32
-static LPCTSTR kRegistryPath = TEXT("Software\\7-zip");
+static LPCTSTR kRegistryPath = TEXT("Software") TEXT(STRING_PATH_SEPARATOR) TEXT("7-zip");
 static LPCTSTR kProgramPathValue = TEXT("Path");
 static bool ReadPathFromRegistry(HKEY baseKey, CSysString &path)
 {
@@ -71,15 +77,19 @@ static bool ReadPathFromRegistry(HKEY baseKey, CSysString &path)
 CSysString GetBaseFolderPrefixFromRegistry()
 {
   CSysString moduleFolderPrefix = GetLibraryFolderPrefix();
-  NFind::CFileInfo fileInfo;
-  if (NFind::FindFile(moduleFolderPrefix + kMainDll, fileInfo))
-    if (!fileInfo.IsDirectory())
+#ifdef _UNICODE
+  NFind::CFileInfoW fi;
+#else
+  NFind::CFileInfo fi;
+#endif
+  if (NFind::FindFile(moduleFolderPrefix + kMainDll, fi))
+    if (!fi.IsDir())
       return moduleFolderPrefix;
-  if (NFind::FindFile(moduleFolderPrefix + kCodecsFolderName, fileInfo))
-    if (fileInfo.IsDirectory())
+  if (NFind::FindFile(moduleFolderPrefix + kCodecsFolderName, fi))
+    if (fi.IsDir())
       return moduleFolderPrefix;
-  if (NFind::FindFile(moduleFolderPrefix + kFormatsFolderName, fileInfo))
-    if (fileInfo.IsDirectory())
+  if (NFind::FindFile(moduleFolderPrefix + kFormatsFolderName, fi))
+    if (fi.IsDir())
       return moduleFolderPrefix;
   #ifdef _WIN32
   CSysString path;
@@ -96,9 +106,10 @@ typedef UInt32 (WINAPI *GetNumberOfFormatsFunc)(UInt32 *numFormats);
 typedef UInt32 (WINAPI *GetHandlerPropertyFunc)(PROPID propID, PROPVARIANT *value);
 typedef UInt32 (WINAPI *GetHandlerPropertyFunc2)(UInt32 index, PROPID propID, PROPVARIANT *value);
 typedef UInt32 (WINAPI *CreateObjectFunc)(const GUID *clsID, const GUID *iid, void **outObject);
+typedef UInt32 (WINAPI *SetLargePageModeFunc)();
 
 
-static HRESULT GetCoderClass(GetMethodPropertyFunc getMethodProperty, UInt32 index, 
+static HRESULT GetCoderClass(GetMethodPropertyFunc getMethodProperty, UInt32 index,
     PROPID propId, CLSID &clsId, bool &isAssigned)
 {
   NWindows::NCOM::CPropVariant prop;
@@ -143,8 +154,8 @@ HRESULT CCodecs::LoadCodecs()
 }
 
 static HRESULT ReadProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, NCOM::CPropVariant &prop)
 {
   if (getProp2)
@@ -153,8 +164,8 @@ static HRESULT ReadProp(
 }
 
 static HRESULT ReadBoolProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, bool &res)
 {
   NCOM::CPropVariant prop;
@@ -167,8 +178,8 @@ static HRESULT ReadBoolProp(
 }
 
 static HRESULT ReadStringProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, UString &res)
 {
   NCOM::CPropVariant prop;
@@ -184,11 +195,11 @@ static HRESULT ReadStringProp(
 
 static const unsigned int kNumArcsMax = 32;
 static unsigned int g_NumArcs = 0;
-static const CArcInfo *g_Arcs[kNumArcsMax]; 
-void RegisterArc(const CArcInfo *arcInfo) 
-{ 
+static const CArcInfo *g_Arcs[kNumArcsMax];
+void RegisterArc(const CArcInfo *arcInfo)
+{
   if (g_NumArcs < kNumArcsMax)
-    g_Arcs[g_NumArcs++] = arcInfo; 
+    g_Arcs[g_NumArcs++] = arcInfo;
 }
 
 static void SplitString(const UString &srcString, UStringVector &destStrings)
@@ -302,6 +313,7 @@ HRESULT CCodecs::LoadFormats()
 #ifdef NEW_FOLDER_INTERFACE
 void CCodecLib::LoadIcons()
 {
+#ifdef _WIN32
   UString iconTypes = MyLoadStringW((HMODULE)Lib, kIconTypesResId);
   UStringVector pairs;
   SplitString(iconTypes, pairs);
@@ -320,20 +332,30 @@ void CCodecLib::LoadIcons()
     iconPair.Ext = s.Left(pos);
     IconPairs.Add(iconPair);
   }
+#endif // #ifdef _WIN32
 }
 
 int CCodecLib::FindIconIndex(const UString &ext) const
 {
+#ifdef _WIN32
   for (int i = 0; i < IconPairs.Size(); i++)
   {
     const CIconPair &pair = IconPairs[i];
     if (ext.CompareNoCase(pair.Ext) == 0)
       return pair.IconIndex;
   }
+#endif // #ifdef _WIN32
   return -1;
 }
 #endif
 
+#ifdef _7ZIP_LARGE_PAGES
+extern "C"
+{
+  extern SIZE_T g_LargePageSize;
+}
+#endif
+
 HRESULT CCodecs::LoadDll(const CSysString &dllPath)
 {
 #ifdef _WIN32
@@ -355,6 +377,16 @@ HRESULT CCodecs::LoadDll(const CSysString &dllPath)
     #ifdef NEW_FOLDER_INTERFACE
     lib.LoadIcons();
     #endif
+
+    #ifdef _7ZIP_LARGE_PAGES
+    if (g_LargePageSize != 0)
+    {
+      SetLargePageModeFunc setLargePageMode = (SetLargePageModeFunc)lib.Lib.GetProcAddress("SetLargePageMode");
+      if (setLargePageMode != 0)
+        setLargePageMode();
+    }
+    #endif
+
     lib.CreateObject = (CreateObjectFunc)lib.Lib.GetProcAddress("CreateObject");
     if (lib.CreateObject != 0)
     {
@@ -376,13 +408,18 @@ HRESULT CCodecs::LoadDll(const CSysString &dllPath)
 
 HRESULT CCodecs::LoadDllsFromFolder(const CSysString &folderPrefix)
 {
+#ifdef _UNICODE
+  NFile::NFind::CEnumeratorW enumerator(folderPrefix + CSysString(TEXT("*")));
+  NFile::NFind::CFileInfoW fi;
+#else
   NFile::NFind::CEnumerator enumerator(folderPrefix + CSysString(TEXT("*")));
-  NFile::NFind::CFileInfo fileInfo;
-  while (enumerator.Next(fileInfo))
+  NFile::NFind::CFileInfo fi;
+#endif
+  while (enumerator.Next(fi))
   {
-    if (fileInfo.IsDirectory())
+    if (fi.IsDir())
       continue;
-    RINOK(LoadDll(folderPrefix + fileInfo.Name));
+    RINOK(LoadDll(folderPrefix + fi.Name));
   }
   return S_OK;
 }
@@ -428,14 +465,16 @@ HRESULT CCodecs::Load()
   return S_OK;
 }
 
-int CCodecs::FindFormatForArchiveName(const UString &archivePath) const
+#ifndef _SFX
+
+int CCodecs::FindFormatForArchiveName(const UString &arcPath) const
 {
-  int slashPos1 = archivePath.ReverseFind(L'\\');
-  int slashPos2 = archivePath.ReverseFind(L'.');
-  int dotPos = archivePath.ReverseFind(L'.');
+  int slashPos1 = arcPath.ReverseFind(WCHAR_PATH_SEPARATOR);
+  int slashPos2 = arcPath.ReverseFind(L'.');
+  int dotPos = arcPath.ReverseFind(L'.');
   if (dotPos < 0 || dotPos < slashPos1 || dotPos < slashPos2)
     return -1;
-  UString ext = archivePath.Mid(dotPos + 1);
+  UString ext = arcPath.Mid(dotPos + 1);
   for (int i = 0; i < Formats.Size(); i++)
   {
     const CArcInfoEx &arc = Formats[i];
@@ -449,46 +488,74 @@ int CCodecs::FindFormatForArchiveName(const UString &archivePath) const
   return -1;
 }
 
+int CCodecs::FindFormatForExtension(const UString &ext) const
+{
+  if (ext.IsEmpty())
+    return -1;
+  for (int i = 0; i < Formats.Size(); i++)
+    if (Formats[i].FindExtension(ext) >= 0)
+      return i;
+  return -1;
+}
+
 int CCodecs::FindFormatForArchiveType(const UString &arcType) const
 {
   for (int i = 0; i < Formats.Size(); i++)
-  {
-    const CArcInfoEx &arc = Formats[i];
-    if (!arc.UpdateEnabled)
-      continue;
-    if (arc.Name.CompareNoCase(arcType) == 0)
+    if (Formats[i].Name.CompareNoCase(arcType) == 0)
       return i;
-  }
   return -1;
 }
 
+bool CCodecs::FindFormatForArchiveType(const UString &arcType, CIntVector &formatIndices) const
+{
+  formatIndices.Clear();
+  for (int pos = 0; pos < arcType.Length();)
+  {
+    int pos2 = arcType.Find('.', pos);
+    if (pos2 < 0)
+      pos2 = arcType.Length();
+    const UString name = arcType.Mid(pos, pos2 - pos);
+    int index = FindFormatForArchiveType(name);
+    if (index < 0 && name != L"*")
+    {
+      formatIndices.Clear();
+      return false;
+    }
+    formatIndices.Add(index);
+    pos = pos2 + 1;
+  }
+  return true;
+}
+
+#endif
+
 #ifdef EXTERNAL_CODECS
 
 #ifdef EXPORT_CODECS
 extern unsigned int g_NumCodecs;
 STDAPI CreateCoder2(bool encode, UInt32 index, const GUID *iid, void **outObject);
 STDAPI GetMethodProperty(UInt32 codecIndex, PROPID propID, PROPVARIANT *value);
-// STDAPI GetNumberOfMethods(UINT32 *numCodecs);
+// STDAPI GetNumberOfMethods(UInt32 *numCodecs);
 #endif
 
-STDMETHODIMP CCodecs::GetNumberOfMethods(UINT32 *numMethods)
+STDMETHODIMP CCodecs::GetNumberOfMethods(UInt32 *numMethods)
 {
-  *numMethods = 
+  *numMethods =
       #ifdef EXPORT_CODECS
-      g_NumCodecs + 
+      g_NumCodecs +
       #endif
       Codecs.Size();
   return S_OK;
 }
 
-STDMETHODIMP CCodecs::GetProperty(UINT32 index, PROPID propID, PROPVARIANT *value)
+STDMETHODIMP CCodecs::GetProperty(UInt32 index, PROPID propID, PROPVARIANT *value)
 {
   #ifdef EXPORT_CODECS
   if (index < g_NumCodecs)
     return GetMethodProperty(index, propID, value);
   #endif
 
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -511,13 +578,13 @@ STDMETHODIMP CCodecs::GetProperty(UINT32 index, PROPID propID, PROPVARIANT *valu
   return Libs[ci.LibIndex].GetMethodProperty(ci.CodecIndex, propID, value);
 }
 
-STDMETHODIMP CCodecs::CreateDecoder(UINT32 index, const GUID *iid, void **coder)
+STDMETHODIMP CCodecs::CreateDecoder(UInt32 index, const GUID *iid, void **coder)
 {
   #ifdef EXPORT_CODECS
   if (index < g_NumCodecs)
     return CreateCoder2(false, index, iid, coder);
   #endif
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -527,13 +594,13 @@ STDMETHODIMP CCodecs::CreateDecoder(UINT32 index, const GUID *iid, void **coder)
   return S_OK;
 }
 
-STDMETHODIMP CCodecs::CreateEncoder(UINT32 index, const GUID *iid, void **coder)
+STDMETHODIMP CCodecs::CreateEncoder(UInt32 index, const GUID *iid, void **coder)
 {
   #ifdef EXPORT_CODECS
   if (index < g_NumCodecs)
     return CreateCoder2(true, index, iid, coder);
   #endif
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -544,7 +611,7 @@ STDMETHODIMP CCodecs::CreateEncoder(UINT32 index, const GUID *iid, void **coder)
 }
 
 HRESULT CCodecs::CreateCoder(const UString &name, bool encode, CMyComPtr<ICompressCoder> &coder) const
-{ 
+{
   for (int i = 0; i < Codecs.Size(); i++)
   {
     const CDllCodecInfo &codec = Codecs[i];
@@ -571,7 +638,7 @@ int CCodecs::GetCodecLibIndex(UInt32 index)
     return -1;
   #endif
   #ifdef EXTERNAL_CODECS
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -595,7 +662,7 @@ bool CCodecs::GetCodecEncoderIsAssigned(UInt32 index)
   }
   #endif
   #ifdef EXTERNAL_CODECS
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
