@@ -6,12 +6,11 @@
 #include "FileName.h"
 #include "FileFind.h"
 #include "Defs.h"
-#ifndef _UNICODE
 #include "../Common/StringConvert.h"
-#endif
 
 #define NEED_NAME_WINDOWS_TO_UNIX
 #include "myPrivate.h"
+#include "Windows/Synchronization.h"
 
 #include <unistd.h> // rmdir
 #include <errno.h>
@@ -25,12 +24,122 @@
 // #define TRACEN(u) u;
 #define TRACEN(u)  /* */
 
-#ifdef HAVE_LSTAT
-extern int global_use_lstat;
+class Umask
+{
+  public:
+  mode_t  current_umask;
+  mode_t  mask;
+  Umask() {
+    current_umask = umask (0);  /* get and set the umask */
+    umask(current_umask);	/* restore the umask */
+    mask = 0777 & (~current_umask);
+  } 
+};
+
+static Umask gbl_umask;
+
+#ifdef _UNICODE
+AString nameWindowToUnix2(LPCWSTR name) // FIXME : optimization ?
+{
+   AString astr = UnicodeStringToMultiByte(name);
+   return AString(nameWindowToUnix((const char *)astr));
+}
 #endif
 
 extern BOOLEAN WINAPI RtlTimeToSecondsSince1970( const LARGE_INTEGER *Time, DWORD *Seconds );
 
+#ifdef _UNICODE
+DWORD WINAPI GetFullPathName( LPCTSTR name, DWORD len, LPTSTR buffer, LPTSTR *lastpart ) { // FIXME
+  if (name == 0) return 0;
+
+  DWORD name_len = lstrlen(name);
+
+  if (name[0] == '/') {
+    DWORD ret = name_len+2;
+    if (ret >= len) {
+      TRACEN((printf("GetFullPathNameA(%ls,%d,)=0000 (case 0)\n",name, (int)len)))
+      return 0;
+    }
+    lstrcpy(buffer,L"c:");
+    lstrcat(buffer,name);
+
+    *lastpart=buffer;
+    TCHAR *ptr=buffer;
+    while (*ptr) {
+      if (*ptr == '/')
+        *lastpart=ptr+1;
+      ptr++;
+    }
+    TRACEN((printf("GetFullPathNameA(%s,%d,%ls,%ls)=%d\n",name, (int)len,buffer, *lastpart,(int)ret)))
+    return ret;
+  }
+  if (isascii(name[0]) && (name[1] == ':')) { // FIXME isascii
+    DWORD ret = name_len;
+    if (ret >= len) {
+      TRACEN((printf("GetFullPathNameA(%ls,%d,)=0000 (case 1)\n",name, (int)len)))
+      return 0;
+    }
+    lstrcpy(buffer,name);
+
+    *lastpart=buffer;
+    TCHAR *ptr=buffer;
+    while (*ptr) {
+      if (*ptr == '/')
+        *lastpart=ptr+1;
+      ptr++;
+    }
+    TRACEN((printf("GetFullPathNameA(%sl,%d,%ls,%ls)=%d\n",name, (int)len,buffer, *lastpart,(int)ret)))
+    return ret;
+  }
+
+  // name is a relative pathname.
+  //
+  if (len < 2) {
+    TRACEN((printf("GetFullPathNameA(%s,%d,)=0000 (case 2)\n",name, (int)len)))
+    return 0;
+  }
+
+  DWORD ret = 0;
+  char begin[MAX_PATHNAME_LEN];
+  /* DWORD begin_len = GetCurrentDirectoryA(MAX_PATHNAME_LEN,begin); */
+  DWORD begin_len = 0;
+  begin[0]='c';
+  begin[1]=':';
+  char * cret = getcwd(begin+2, MAX_PATHNAME_LEN - 3);
+  if (cret) {
+    begin_len = strlen(begin);
+  }
+   
+  if (begin_len >= 1) {
+    //    strlen(begin) + strlen("/") + strlen(name)
+    ret = begin_len     +    1        + name_len;
+
+    if (ret >= len) {
+      TRACEN((printf("GetFullPathNameA(%s,%d,)=0000 (case 4)\n",name, (int)len)))
+      return 0;
+    }
+    UString wbegin = GetUnicodeString(begin);
+    lstrcpy(buffer,wbegin);
+    lstrcat(buffer,L"/");
+    lstrcat(buffer,name);
+
+    *lastpart=buffer + begin_len + 1;
+    TCHAR *ptr=buffer;
+    while (*ptr) {
+      if (*ptr == '/')
+        *lastpart=ptr+1;
+      ptr++;
+    }
+    TRACEN((printf("GetFullPathNameA(%s,%d,%s,%s)=%d\n",name, (int)len,buffer, *lastpart,(int)ret)))
+  } else {
+    ret = 0;
+    TRACEN((printf("GetFullPathNameA(%s,%d,)=0000 (case 5)\n",name, (int)len)))
+  }
+  return ret;
+}
+
+#endif
+
 DWORD WINAPI GetFullPathName( LPCSTR name, DWORD len, LPSTR buffer, LPSTR *lastpart ) {
   if (name == 0) return 0;
 
@@ -132,6 +241,21 @@ static BOOL WINAPI RemoveDirectory(LPCSTR path) {
   }
   return TRUE;
 }
+#ifdef _UNICODE
+static BOOL WINAPI RemoveDirectory(LPCWSTR path) {
+  if (!path || !*path) {
+    SetLastError(ERROR_PATH_NOT_FOUND);
+    return FALSE;
+  }
+  AString name = nameWindowToUnix2(path);
+  TRACEN((printf("RemoveDirectoryA(%s)\n",(const char *)name)))
+
+  if (rmdir( (const char *)name ) != 0) {
+    return FALSE;
+  }
+  return TRUE;
+}
+#endif
 
 static int copy_fd(int fin,int fout)
 {
@@ -186,9 +310,68 @@ static BOOL CopyFile(const char *src,const char *dst)
   return FALSE;
 }
 
+/*****************************************************************************************/
+
+
+namespace NWindows {
+namespace NFile {
+namespace NDirectory {
+
+
+bool MySetCurrentDirectory(LPCWSTR wpath)
+{
+   AString path = UnicodeStringToMultiByte(wpath);
+
+   return chdir((const char*)path) == 0;
+}
+
+#ifdef _UNICODE
+bool GetOnlyName(LPCTSTR fileName, CSysString &resultName)
+{
+  int index;
+  if (!MyGetFullPathName(fileName, resultName, index))
+    return false;
+  resultName = resultName.Mid(index);
+  return true;
+}
+
+bool GetOnlyDirPrefix(LPCTSTR fileName, CSysString &resultName)
+{
+  int index;
+  if (!MyGetFullPathName(fileName, resultName, index))
+    return false;
+  resultName = resultName.Left(index);
+  return true;
+}
+#endif
+
+
+bool MyGetCurrentDirectory(CSysString &resultPath)
+{
+  char begin[MAX_PATHNAME_LEN];
+  begin[0]='c';
+  begin[1]=':';
+  char * cret = getcwd(begin+2, MAX_PATHNAME_LEN - 3);
+  if (cret)
+  {
+#ifdef _UNICODE
+    resultPath = GetUnicodeString(begin);
+#else
+    resultPath = begin;
+#endif
+    return true;
+  }
+  return false;
+}
+
 bool MyMoveFile( LPCTSTR fn1, LPCTSTR fn2 ) {
+#ifdef _UNICODE
+  AString src = nameWindowToUnix2(fn1);
+  AString dst = nameWindowToUnix2(fn2);
+#else
   const char * src = nameWindowToUnix(fn1);
   const char * dst = nameWindowToUnix(fn2);
+#endif
 
   TRACEN((printf("MoveFileW(%s,%s)\n",src,dst)))
 
@@ -203,7 +386,8 @@ bool MyMoveFile( LPCTSTR fn1, LPCTSTR fn2 ) {
       struct stat info_file;
       ret = stat(src,&info_file);
       if (ret == 0) {
-        ret = chmod(dst,info_file.st_mode);
+	TRACEN((printf("##DBG chmod-1(%s,%o)\n",dst,(unsigned)info_file.st_mode & gbl_umask.mask)))
+        ret = chmod(dst,info_file.st_mode & gbl_umask.mask);
       }
       if (ret == 0) {
          ret = unlink(src);
@@ -215,13 +399,6 @@ bool MyMoveFile( LPCTSTR fn1, LPCTSTR fn2 ) {
   return true;
 }
 
-/*****************************************************************************************/
-
-
-namespace NWindows {
-namespace NFile {
-namespace NDirectory {
-
 bool MyRemoveDirectory(LPCTSTR pathName)
 {
   return BOOLToBool(::RemoveDirectory(pathName));
@@ -285,7 +462,7 @@ bool MyRemoveDirectory(LPCWSTR pathName)
 bool MyMoveFile(LPCWSTR existFileName, LPCWSTR newFileName)
 {  
   UINT codePage = CP_ACP;
-  return ::MyMoveFile(UnicodeStringToMultiByte(existFileName, codePage), UnicodeStringToMultiByte(newFileName, codePage));
+  return MyMoveFile(UnicodeStringToMultiByte(existFileName, codePage), UnicodeStringToMultiByte(newFileName, codePage));
 }
 #endif
 
@@ -314,7 +491,11 @@ bool MySetFileAttributes(LPCTSTR fileName, DWORD fileAttributes)
     TRACEN((printf("MySetFileAttributes(NULL,%d) : false-1\n",fileAttributes)))
     return false;
   }
+#ifdef _UNICODE
+  AString name = nameWindowToUnix2(fileName);
+#else
   const char * name = nameWindowToUnix(fileName);
+#endif
   struct stat stat_info;
 #ifdef HAVE_LSTAT
   if (global_use_lstat) {
@@ -342,11 +523,13 @@ bool MySetFileAttributes(LPCTSTR fileName, DWORD fileAttributes)
      } else
 #endif
      if (S_ISREG(stat_info.st_mode)) {
-       chmod(name,stat_info.st_mode);
+       TRACEN((printf("##DBG chmod-2(%s,%o)\n",name,(unsigned)stat_info.st_mode & gbl_umask.mask)))
+       chmod(name,stat_info.st_mode & gbl_umask.mask);
      } else if (S_ISDIR(stat_info.st_mode)) {
        // user/7za must be able to create files in this directory
        stat_info.st_mode |= (S_IRUSR | S_IWUSR | S_IXUSR);
-       chmod(name,stat_info.st_mode);
+       TRACEN((printf("##DBG chmod-3(%s,%o)\n",name,(unsigned)stat_info.st_mode & gbl_umask.mask)))
+       chmod(name,stat_info.st_mode & gbl_umask.mask);
      }
 #ifdef HAVE_LSTAT
   } else if (!S_ISLNK(stat_info.st_mode)) {
@@ -354,17 +537,17 @@ bool MySetFileAttributes(LPCTSTR fileName, DWORD fileAttributes)
 #else
   } else {
 #endif
-    if (fileAttributes & FILE_ATTRIBUTE_READONLY) {
-      if(!S_ISDIR(stat_info.st_mode)) {
-        /* FILE_ATTRIBUTE_READONLY ignored for directory. */
-        stat_info.st_mode &= ~0222; /* octal!, clear write permission bits */
-      }
-    } else {
-      /* add write permission */
-      stat_info.st_mode |= (0600 | ((stat_info.st_mode & 044) >> 1)) ;
-    }
 
-    chmod(name,stat_info.st_mode);
+    /* Only Windows Attributes */
+    if( S_ISDIR(stat_info.st_mode)) {
+       /* Remark : FILE_ATTRIBUTE_READONLY ignored for directory. */
+       TRACEN((printf("##DBG chmod-4(%s,%o)\n",name,(unsigned)stat_info.st_mode & gbl_umask.mask)))
+       chmod(name,stat_info.st_mode & gbl_umask.mask);
+    } else {
+       if (fileAttributes & FILE_ATTRIBUTE_READONLY) stat_info.st_mode &= ~0222; /* octal!, clear write permission bits */
+       TRACEN((printf("##DBG chmod-5(%s,%o)\n",name,(unsigned)stat_info.st_mode & gbl_umask.mask)))
+       chmod(name,stat_info.st_mode & gbl_umask.mask);
+    }
   }
   TRACEN((printf("MySetFileAttributes(%s,%d) : true\n",name,fileAttributes)))
 
@@ -378,7 +561,11 @@ bool MyCreateDirectory(LPCTSTR pathName)
     return false;
   }
 
+#ifdef _UNICODE
+  AString name = nameWindowToUnix2(pathName);
+#else
   const char * name = nameWindowToUnix(pathName);
+#endif
   bool bret = false;
   if (mkdir( name, 0700 ) == 0) bret = true;
 
@@ -411,11 +598,13 @@ bool CreateComplexDirectory(LPCTSTR _aPathName)
       break;
     if(::GetLastError() == ERROR_ALREADY_EXISTS)
     {
+#ifdef _WIN32 // FIXED for supporting symbolic link instead of a directory
       NFind::CFileInfo fileInfo;
       if (!NFind::FindFile(pathName, fileInfo)) // For network folders
         return true;
-      if (!fileInfo.IsDirectory())
+      if (!fileInfo.IsDir())
         return false;
+#endif
       break;
     }
     pos = pathName.ReverseFind(TEXT(CHAR_PATH_SEPARATOR));
@@ -457,11 +646,13 @@ bool CreateComplexDirectory(LPCWSTR _aPathName)
       break;
     if(::GetLastError() == ERROR_ALREADY_EXISTS)
     {
+#ifdef _WIN32 // FIXED for supporting symbolic link instead of a directory
       NFind::CFileInfoW fileInfo;
       if (!NFind::FindFile(pathName, fileInfo)) // For network folders
         return true;
-      if (!fileInfo.IsDirectory())
+      if (!fileInfo.IsDir())
         return false;
+#endif
       break;
     }
     pos = pathName.ReverseFind(WCHAR_PATH_SEPARATOR);
@@ -491,7 +682,11 @@ bool DeleteFileAlways(LPCTSTR name)
     SetLastError(ERROR_PATH_NOT_FOUND);
     return false;
   }
+#ifdef _UNICODE
+   AString unixname = nameWindowToUnix2(name);
+#else
    const char * unixname = nameWindowToUnix(name);
+#endif
    bool bret = false;
    if (remove(unixname) == 0) bret = true;
    TRACEN((printf("DeleteFileAlways(%s)=%d\n",unixname,(int)bret)))
@@ -595,11 +790,13 @@ bool MySearchPath(LPCWSTR path, LPCWSTR fileName, LPCWSTR extension, UString &re
   return false;
 }
 
+#ifndef _UNICODE
 bool MyGetTempPath(CSysString &path)
 {
   path = "c:/tmp/"; // final '/' is needed
   return true;
 }
+#endif
 
 bool MyGetTempPath(UString &path)
 {
@@ -607,18 +804,33 @@ bool MyGetTempPath(UString &path)
   return true;
 }
 
+static NSynchronization::CCriticalSection g_CountCriticalSection;
+
 UINT CTempFile::Create(LPCTSTR dirPath, LPCTSTR prefix, CSysString &resultPath)
 {
+  static int memo_count = 0;
+  int count;
+
+  g_CountCriticalSection.Enter();
+  count = memo_count++;
+  g_CountCriticalSection.Leave();
+
   Remove();
 /* UINT number = ::GetTempFileName(dirPath, prefix, 0, path.GetBuffer(MAX_PATH)); */
   UINT number = (UINT)getpid();
-  sprintf(resultPath.GetBuffer(MAX_PATH),"%s%s%d.tmp",dirPath,prefix,(int)number);
+  TCHAR * buf = resultPath.GetBuffer(MAX_PATH);
+#ifdef _UNICODE
+  swprintf(buf,MAX_PATH,L"%s%s#%x@%x.tmp",dirPath,prefix,(unsigned)number,count);
+#else
+  snprintf(buf,MAX_PATH,"%s%s#%x@%x.tmp",dirPath,prefix,(unsigned)number,count);
+#endif
+ 
+  buf[MAX_PATH-1]=0;
   resultPath.ReleaseBuffer();
-  if(number != 0)
-  {
-    _fileName = resultPath;
-    _mustBeDeleted = true;
-  }
+  
+  _fileName = resultPath;
+  _mustBeDeleted = true;
+ 
   return number;
 }
 
